//------------------------------------------------------------------
// Create by Robert F. Nee.  Use freely.
//
// See http://en.wikipedia.org/wiki/Nose_cone_design#Secant_ogive for more info
//
// Estes Jupiter C (#1976) nose cone stack and connector fairing parts
//
//------------------------------------------------------------------

include <Estes.scad>;
include <rocketry.scad>
include <fins.scad>

//------------------------------------------------------------------
// Functions to extract segment height and total height from h vectors
function stack (a, s) =
	a[s] - a[0]; 

function height (a, s) =
	a[s] - a[max (s - 1, 0)]; 

//------------------------------------------------------------------
// Draw a tapered leading edge suitable for mating with a tapered fin

module thruster ()
{
	// Thrusters modeled as fins
	root_thick		= 0.315;	// thruster root thickness
	tip_thick			= 0.260;	// thruster tip thickness
	width			= 0.090;	// thruster width
	root_height		= 0.160;	// thruster root height
	tip_height		= 0.100;	// thruster tip height
	root_point		= 0.430;	// thruster root point height
	tip_point			= 0.260;	// thruster tip height

	// Ten points
	points = [
		[-tip_thick/2, -width, 0],			// P0
		[-tip_thick/2, -width, tip_height],	// P1 
		[0, -width, tip_height + tip_point],	// P2 
		[tip_thick/2, -width, tip_height],		// P3 
		[tip_thick/2, -width, 0],			// P4
		[-root_thick/2, 0, 0],				// P5
		[-root_thick/2, 0, root_height],		// P6 
		[0, 0, root_height + root_point],		// P7 
		[root_thick/2, 0, root_height],		// P8 
		[root_thick/2, 0, 0],				// P9
	];	

	// 9 faces
	faces = [
		[0, 1, 2, 3, 4],			// front
		[9, 8, 7, 6, 5],			// back
		[0, 4, 9, 5],				// rear
		[0, 5, 6, 1],				// left
		[4, 3, 8, 9],				// right
		[1, 6, 7],				// lead1
		[2, 1, 7],				// lead2
		[3, 2, 7],				// lead3
		[3, 7, 8]					// lead4
	];

	polyhedron (points, faces);
}

//------------------------------------------------------------------

module cone ()
{
	h = [
		0,
		1.460,					// base portion
		3.620,					// first cone
		4.000,					// second cone
		4.010,					// section between cone 2 and 3
		4.390,					// third cone
		4.460,					// section between cone 3 and 4
		4.560					// forth cone
	];
	
	h = [
		1.829,
		3.256,					// base portion
		5.405,					// first cone
		5.773,					// second cone
		5.843,					// section between cone 2 and 3
		6.229,					// third cone
		6.305,					// section between cone 3 and 4
		6.390					// forth cone
	];
	
	d = [
		[od,		od],
		[od,		0.930],
		[0.930,	0.500],
		[0.500,	0.500],
		[0.500,	0.415],
		[0.290,	0.285],
		[0.285,	0]
	];	
	
	// shoulder
	cylinder (h=sh, d=id);
	
	for (i = [0 : len (d) - 1]) {
		translate ([0, 0, sh + stack (h, i)])
			cylinder (h=height (h, i + 1) + 0.001, d1=d[i][0], d2=d[i][1]);
	}
	
	
	// Thruster placement dimensions
	thruster_root_thick		= 0.315;	// thruster root thickness
	thruster_offset		= 0.100;	// thruster offset from shoulder
	
	thruster_inset = fin_inset (od, thruster_root_thick);
	
	// Thrusters
	for (i = [0:3]) {
		rotate ([0, 0, i * 90])
			translate ([0, -(od/2 - thruster_inset), sh + thruster_offset])
				thruster ();
	}
}

//------------------------------------------------------------------
// Bucket - whole bucket

module bucket ()
{
	h = [
		0,
		1.390,					// base portion
		1.430,					// first lip
		1.480,					// second lip
		1.570,					// extension
		1.610,					// bullnose
		2.180					// cone
	];
	
	d = [
		[1.015,	1.035],
		[1.140,	1.140],
		[1.040,	1.040],
		[1.000,	1.000],
		[1.000,	0.920],
		[0.430,	0.215]
	];	
	
	difference () {
		union () {
			for (i = [0 : len (d) - 1]) {
				translate ([0, 0, stack (h, i)])
					cylinder (h=height (h, i + 1) + 0.001, d1=d[i][0], d2=d[i][1]);
			}
		}

		// Holes
		translate ([0, 0, -0.001]) {
			cylinder (h=3, d=0.170);
			cylinder (h=0.436, d=0.500);
			cylinder (h=h[1], d=0.300);			
		}
	}
	
	// Rivets
	translate ([0, 0, stack (h, 5)])
	for (i = [0 : 10]) {
		rotate ([0, 0, i * 360/11])
			translate ([0.740/2, 0, 0])
				cylinder (h=0.070, d=0.080);
	}

	// Bullnose
	r1 = (d[4][0] - d[4][1])/2;
	r2 = (d[4][0])/2 - r1;

	translate ([0, 0, stack (h, 4)])
	rotate_extrude(convexity = 10)
		translate([r2, 0, 0])
			circle(r = r1);

}

//------------------------------------------------------------------
// Bucket - body portion.  Flip for printing

module bucket_body ()
{
	h = [
		0,
		1.390,					// base portion
		1.430,					// first lip
		1.480,					// second lip
		1.570,					// extension
		1.610,					// bullnose
		2.180					// cone
	];
	
	d = [
		[1.015,	1.035],
		[1.140,	1.140],
		[1.040,	1.040],
		[1.000,	1.000],
		[1.000,	0.920],
		[0.430,	0.215]
	];	
	
	difference () {
		union () {
			for (i = [0 : 1]) {
				translate ([0, 0, stack (h, i)])
					cylinder (h=height (h, i + 1) + 0.001, d1=d[i][0], d2=d[i][1]);
			}
		}

		// Holes
		translate ([0, 0, -0.001]) {
			cylinder (h=3, d=0.170);
			cylinder (h=0.436, d=0.500);
			cylinder (h=h[1], d=0.300);			
		}
	}
}

//------------------------------------------------------------------
// Bucket - cap portion

module bucket_cap ()
{
	h = [
		1.430,					// first lip
		1.480,					// second lip
		1.570,					// extension
		1.610,					// bullnose
		2.180					// cone
	];
	
	d = [
		[1.040,	1.040],
		[1.000,	1.000],
		[1.000,	0.920],
		[0.430,	0.215]
	];	

	difference () {
		union () {
			for (i = [0 : len (d) - 1]) {
				translate ([0, 0, stack (h, i)])
					cylinder (h=height (h, i + 1) + 0.001, d1=d[i][0], d2=d[i][1]);
			}
		}

		// Holes
		translate ([0, 0, -0.001]) {
			cylinder (h=3, d=0.170);
		}
	}
	
	// Rivets
	translate ([0, 0, stack (h, 3)])
	for (i = [0 : 10]) {
		rotate ([0, 0, i * 360/11])
			translate ([0.740/2, 0, 0])
				cylinder (h=0.070, d=0.080);
	}

	// Bullnose
	r1 = (d[2][0] - d[2][1])/2;
	r2 = (d[2][0])/2 - r1;

	translate ([0, 0, stack (h, 2)])
	rotate_extrude(convexity = 10)
		translate([r2, 0, 0])
			circle(r = r1);

}


//------------------------------------------------------------------

module explorer ()
{
	h = [
		0,
		0.140,
		0.200,
		0.300,
		0.495,
		0.520,
		1.250,
		1.290,
		2.060,
		2.360
	];

	d = [
		[0.150,	0.070],
		[0.070,	0.070],
		[0.110,	0.165],
		[0.165,	0.165],
		[0.200,	0.200],
		[0.170,	0.170],
		[0.170,	0.185],
		[0.185,	0.185],
		[0.185,	0.070]
	];	
	
	for (i = [0 : len (d) - 1]) {
		translate ([0, 0, stack (h, i)])
			cylinder (h=height (h, i + 1) + 0.001, d1=d[i][0], d2=d[i][1]);
	}
}

//------------------------------------------------------------------
// Connector Fairing

module wedge (l, w, h)
{
	points = [
		[ -w/2, 0, 0 ],
		[ -w/2, l, 0 ],
		[ w/2, l, 0 ],
		[ w/2, 0, 0 ],
 	 	[ -w/2, 0, h ],
		[ w/2, 0, h ]
	];

	faces = [
		[ 3, 2, 1, 0 ],
		[ 0, 1, 4 ],
		[ 5, 2, 3 ],
		[ 0, 4, 5, 3 ],
		[ 1, 2, 5, 4 ]
	];

	polyhedron (points, faces);
}

module fairing ()
{
	width	= 0.380;
	extra	= 0.000;
	length	= 0.390;
	height	= 0.145;
	thick	= 0.030;
	d		= ST20[OD];
	
	difference () {
		union () {
			translate ([-(width + extra)/2, 0, 0]) 
				cube ([width + extra, length + extra, thick]);
			translate ([0, 0, thick])
				wedge (length - 0.040, width - 0.080, height - thick);
		}
	
		// interior
		translate ([0, -0.001, -0.001])
			wedge (length - 0.040, width - 0.080 - 2 * thick, height - thick);
	
		// body tube relief
		translate ([0, 0, -d/2 + thick/3])
		rotate ([-90, 0, 0])
		cylinder (h = 1, d = d, $fn = 100);
	}
}

//------------------------------------------------------------------
// Dimension are in inches

$fn = 100; 					// Number of fragments used to generate circles
d1 = 0.930;					// Diameter at top first cone
d2 = 0.500;					// diameter of top of second cone
d3 = 0.415;					// diameter of top of third cone
d4 = 0.285;					// Diameter of base of fourth cone

od = ST20[OD];					// OD of base of cone in inches
id = ST20[ID] - 0.004;			// ID of BT-20 plus some room for shoulder
sh = 0.750;					// Height of shoulder in inches


//------------------------------------------------------------------
// stacked view

module stacked () {
	cone();
	
	translate ([0, 0, 4.70])
	bucket_body ();
	
	translate ([0, 0, 6.13])
	bucket_cap ();
	
	translate ([0, 0, 6.39])
	explorer ();
}

//------------------------------------------------------------------
// Exploded view

module exploded ()
{
	cone();
	
	translate ([0, 0, 5.50])
	bucket_body ();
	
	translate ([0, 0, 7.25])
	bucket_cap ();
	
	translate ([0, 0, 8.20])
	explorer ();
}

//------------------------------------------------------------------
//

stacked ();
exploded ();
cone();
bucket_body ();
bucket_cap ();
explorer ();
!fairing ();
